/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

//
// gui_draw.c
//

#include "gui_local.h"

/*
=============================================================================

	WINDOW RENDERING

=============================================================================
*/

/*
==================
GUI_DrawWindows
==================
*/
static vec4_t	debugBounds[6] = {
	{ 1, 0, 0, 0.25f },
	{ 0, 1, 0, 0.25f },
	{ 1, 1, 0, 0.25f },
	{ 0, 0, 1, 0.25f },
	{ 0, 1, 1, 0.25f },
	{ 1, 0, 1, 0.25f },
};
static int numDebug;
static void GUI_DrawWindows_r (guiWindow_t *window)
{
	guiWindow_t	*child;
	uint32		i;

	if (!window->visible)
		return;

	// Draw the window
	if (window->flags & WNFL_FILLCOLOR)
		cgi.R_DrawFill (window->xPos, window->yPos, window->width, window->height, window->generic.fillColor);
	if (window->flags & WNFL_MATERIAL)
		cgi.R_DrawPic (window->generic.material, 0,
						window->xPos, window->yPos,
						window->width, window->height,
						0, 0, window->generic.matScaleX, window->generic.matScaleY,
						window->generic.matColor);

	// Draw bounds
	if (gui_debugBounds->integer == 2 && !(window->flags & WNFL_NOEVENTS))
		cgi.R_DrawFill (window->mins[0], window->mins[1], window->maxs[0], window->maxs[1], debugBounds[(numDebug++)%6]);

	// Recurse down the children
	for (i=0, child=window->childList ; i<window->numChildren ; child++, i++)
		GUI_DrawWindows_r (child);
}
static void GUI_DrawWindows (gui_t *gui)
{
	guiWindow_t	*window;
	uint32		i;

	numDebug = 0;

	// Draw the base GUI
	if (gui->flags & GUIFL_FILLCOLOR)
		cgi.R_DrawFill (gui->xPos, gui->yPos, gui->width, gui->height, gui->generic.fillColor);
	if (gui->flags & GUIFL_MATERIAL)
		cgi.R_DrawPic (gui->generic.material, 0,
						gui->xPos, gui->yPos,
						gui->width, gui->height,
						0, 0, gui->generic.matScaleX, gui->generic.matScaleY,
						gui->generic.matColor);

	// Draw all windows
	for (i=0, window=gui->windowList ; i<gui->numWindows ; window++, i++)
		GUI_DrawWindows_r (window);

	// Debug the bounds of the selected window
	if (gui->cursorWindow && gui_debugBounds->integer == 1)
		cgi.R_DrawFill (gui->cursorWindow->mins[0], gui->cursorWindow->mins[1],
						gui->cursorWindow->maxs[0], gui->cursorWindow->maxs[1],
						debugBounds[(numDebug++)%6]);
}

/*
=============================================================================

	CURSOR RENDERING

=============================================================================
*/

/*
==================
GUI_DrawCursor

Draws the mouse cursor
==================
*/
static void GUI_DrawCursor (gui_t *gui)
{
	if (!gui->cursorShader || gui->flags & GUIFL_NOCURSOR)
		return;

	cgi.R_DrawPic (gui->cursorShader, 0, gui->cursorX, gui->cursorY,
		gui->cursorWidth, gui->cursorHeight, 0, 0, 1, 1, gui->cursorColor);
}

/*
=============================================================================

	MAIN REFRESH

=============================================================================
*/

/*
==================
GUI_RenderGUI
==================
*/
static GUI_r_ScaleGUI (guiWindow_t *window, float xScale, float yScale)
{
	guiWindow_t	*child;
	uint32		i;

	window->xPos = window->rect[0] * xScale;
	window->yPos = window->rect[1] * yScale;
	window->width = window->rect[2] * xScale;
	window->height = window->rect[3] * yScale;
	for (i=0, child=window->childList ; i<window->numChildren ; child++, i++)
		GUI_r_ScaleGUI (child, xScale, yScale);
}
static void GUI_ScaleGUI (gui_t *gui)
{
	float		xScale, yScale;
	guiWindow_t	*window;
	uint32		i;

	// Calculate aspect
	// FIXME: ...
	xScale = 1;
	yScale = 1;

	// Calculate the xPos, yPos, width, and height from the compiled rect
	gui->xPos = gui->rect[0] * xScale;
	gui->yPos = gui->rect[1] * yScale;
	gui->width = gui->rect[2] * xScale;
	gui->height = gui->rect[3] * yScale;
	for (i=0, window=gui->windowList ; i<gui->numWindows ; window++, i++)
		GUI_r_ScaleGUI (window, xScale, yScale);
}


/*
==================
GUI_RenderGUI
==================
*/
void GUI_RenderGUI (struct gui_s *gui)
{
	// Scale the interface
	GUI_ScaleGUI (gui);

	// Run scripted events
	GUI_RunAllEvents (gui);

	// Check for window collision
	GUI_CursorUpdate (gui);

	// Draw the GUI and it's windows
	GUI_DrawWindows (gui);

	// Draw the mouse cursor
	GUI_DrawCursor (gui);
}

void GUI_DrawAllGUIs (void)
{
	gui_t	*gui;
	uint32	i;

	// Render each GUI
	for (i=0, gui=cg_guiState.guiLayers[0] ; i<cg_guiState.guiDepth ; gui++, i++)
		GUI_RenderGUI (gui);
}
